@@ -20,16 +20,11 @@
  * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
  * USE OR OTHER DEALINGS IN THE SOFTWARE. */
 
-/* XXX WTF is this! I shouldn't have to include those first three! FUCK! */
-#include <stdint.h>
-#include <stdlib.h>
-#include "drm.h"
-#include "radeon_drm.h"
-#include "radeon_cs.h"
+#ifndef RADEON_R300_H
+#define RADEON_R300_H
 
-#include "r300_winsys.h"
+#include "radeon_winsys.h"
 
-#include "radeon_buffer.h"
+void radeon_setup_winsys(int fd, struct radeon_winsys* winsys);
 
-struct r300_winsys*
-radeon_create_r300_winsys(int fd, struct radeon_winsys* old_winsys);
+#endif /* RADEON_R300_H */
